////////////////////////////////////////////////////////////////////////////
//
//  PLXADM-Serial.c
//  This program has been modified from the previous work of others to serve
//  as a starting point for the fast implementation of custom protocols for
//  ProLinx customers
//
//  Original work:
//  Rich Reynolds, Vital Computer Solutions
//  (c) 1999, Vital Computer Solutions
//  All Rights Reserved
//
//  Modified by:
//  Ken Hopwood, ProSoft Technology
//  Copyright(c) 2000-2006 ProSoft Technology
//  Date    : 05/09/00  Develop LAS module for UK customer
//  Date    : 06/20/00  Develop GEN module
//	 Date		: 09/12/05  Developed Prolinx version
//
//  Joe Amorntrakul, ProSoft Technology
//  (c) 2005, ProSoft Technology
//  Date    : 04/08/05  Develop Sample module
//
//
//  Environment:	MVI
//						GS-DOS
//						Borland C/C++ Compiler v5.02 (16-bit DOS)
//						Digital Mars C/C++ Compiler v8.49 (16-bit DOS)
//
// ProSoft Technology, Inc. grants you a non-exclusive license to use, modify
// and re-distribute this program provided that this copyright notice and
// license appear on all copies of the software.
//
// Software is provided "AS IS," without a warranty of any kind. ALL EXPRESS OR
// IMPLIED REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
// HEREBY EXCLUDED. THE ENTIRE RISK ARISING OUT OF USING THE SOFTWARE IS ASSUMED
// BY THE LICENSEE.
//
// PROSOFT TECHNOLOGY, INC. AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
// DAMAGES SUFFERED BY LICENSEE OR ANY THIRD PARTY AS A RESULT OF USING OR
// DISTRIBUTING SOFTWARE.  IN NO EVENT WILL PROSOFT TECHNOLOGY, INC. OR ITS
// LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
// INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
// CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
// OR INABILITY TO USE SOFTWARE, EVEN IF PROSOFT TECHNOLOGY, INC. HAS BEEN
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
////////////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <dos.h>
#include <stdlib.h>

#include "PLXADM-Serial.H"

int setup_module(void);
int shutdown_module(void);
int startup(void);

// baud settings for upper ports (1 - 3)
// Use baud rate as offset in table.  Ports 1-3 require a setting of 1/2
// of the normal baud rate due to clock differences
int adjusted_baud[] =
{
   BAUD_4800,    	//BAUD110		0	/* invalid baud */
   BAUD_4800,    	//BAUD150		1	/* invalid baud */
   BAUD_150,     	//BAUD300		2	/* 300 baud */
   BAUD_300,     	//BAUD600		3	/* 600 baud */
   BAUD_600,     	//BAUD1200		4	/* 1200	baud */
   BAUD_1200,    	//BAUD2400		5	/* 2400	baud */
   BAUD_2400,    	//BAUD4800		6	/* 4800	baud */
   BAUD_4800,    	//BAUD9600		7	/* 9600	baud */
   BAUD_9600,    	//BAUD19200		8	/* 19200 baud */
	BAUD_4800,		//BAUD28800		9 	/* 28800 baud */
   BAUD_19200,   	//BAUD38400		10	/* 38400 baud */
   BAUD_28800,   	//BAUD57600		11	/* 57600 baud */
   BAUD_57600,   	//BAUD115200	12	/* 115.2 kbaud */
};

//---------------------- do not edit following code -----------------------
static void interrupt intctl_c(void);
static void interrupt intcrit(void);

// Ctrl-C signal handling function.  When a Ctrl-C is recognized, this function
// is called.  The function ignores the request and must reinstall the signal
// handler for future calls.
static void interrupt intctl_c(void)
{
	return;
}

static void interrupt intcrit(void)
{
	_AX = 0;   // ignore critical error
}
//--------------------- ok to edit following code ---------------------------

void main(void)
{
	BYTE str0[20];			// buffer used for port 0
   int len0;            // buffer length variables used in code
	int sptr0;
   int len;             // buffer length variables used in code

#ifdef __DMC__
	extern int _8087;

	_8087 = 0; /* never use 80x87 Math Coprocessor */
#endif

	/* enable console on COM1 */
	ADM_SetConsolePort(COM1);
	/* set console to 57600 baud */
	ADM_SetConsoleSpeed(COM1, 57600L);

	/* open ADM API and get handle */
	if(ADM_Open(&adm_handle) != ADM_SUCCESS)
	{
		printf("\nFailed to open ADM API... exiting program\n");
		exit(1);
	}

	/* send status message to the status/debug port */
	printf("\nProgram Initialization....\n");
	if(!setup_module())         // Initialize hardware and load drivers
	{
		shutdown_module();      // if fail on init then shutdown
		return;
	}

	sptr0 = 0;

   // Main loop for program ---------------------------------------------------
	for(;;)
	{
		// Port 0 handler logic -------------------------------------------------
		len0 = 10;
		// Get chars from COM port. If LF is found, process string
		if(MVIsp_Gets(COM2, &str0[sptr0], _LF, &len0, TIMEOUT_ASAP) == MVI_SUCCESS)
		{
			str0[sptr0] = '\0';
			// Send message back out port 1 -----------------------------------------
			if(MVIsp_Puts(COM3, &str0[0], '\0', &len, TIMEOUT_ASAP) != MVI_SUCCESS)
			{
				printf("Error write to port 1\n");
			}

			// Send message back out port 2 -----------------------------------------
			if(MVIsp_Puts(COM4, &str0[0], '\0', &len, TIMEOUT_ASAP) != MVI_SUCCESS)
			{
				printf("Error write to port 2\n");
			}

			// Send message back out port 3 -----------------------------------------
			if(MVIsp_Puts(COM5, &str0[0], '\0', &len, TIMEOUT_ASAP) != MVI_SUCCESS)
			{
				printf("Error write to port 3\n");
			}
			sptr0 = 0;
			len0 = 0;
		}
		if(len0 > 0)	// message not yet complete
		{
			sptr0 = sptr0 + len0;		// bump string pointer to next free byte
		}


      // Key Board Input ------------------------------------------------------
   	if(kbhit())
   	{
   		if(getch() == 27)	// test for ESC key
   		{
		      shutdown_module();      // if fail on init then shutdown
		      exit(1);
   		}
   		else
	         printf("Press Esc to Exit.\n\n");
   	}
	}  // End Main loop
}


///////////////////////////////////////////////////////////////////////////////
//
// This function is used to setup the module's hardware and load device
// drivers required by the application.  Most of the items to look at closely
// are going to be related to the serial port configuration value at the top of
// of the function.
//
// Input : None.
// Output: The function returns a value of 1 if successful and 0 if not.
//
///////////////////////////////////////////////////////////////////////////////

int setup_module(void)
{
	int se;
	char port_set = 0;
	char eport_set = 0;

	/******************************************************************
		This section is required for the ADM API to function.
		The names of the structures may be changed to suit the
		developer's needs.  The interface structure must be initialized
		before it can be used.
	*******************************************************************
	*/

	/* initialize structure pointers */
	interface_ptr = &interface;
	MVIsp_GetVersionInfo(&spverinfo);
	ADM_GetVersionInfo(adm_handle, &adm_version);

	//----------- do not edit next 6 lines of code---------------------------
	disable();
	setvect(0x24, intcrit);     // install in ISR handler for CTRL-C
	setvect(0x23, intctl_c);    // install in ISR handler for CTRL-C
	setvect(0x1B, intctl_c);    // install in ISR handler for CTRL-Break
	setcbrk(0);                 // force DOS to ignore Ctrl-C
	enable();
	//-----------------------------------------------------------------------

	// Open serial interface API
	// Set comm parameters

	// Port 0
	printf("Open Communication Port 0 9600 8 1 None....");
	se = MVIsp_Open(COM2, BAUD_9600, PARITY_NONE, WORDLEN8, STOPBITS1);
	if(se != MVI_SUCCESS){
	   printf("Unsuccessfully!\n");
		return 0;
	}
	else
	   printf("Successfully!\n");

	// Port 1
	printf("Open Communication Port 1 9600 8 1 None....");
	se = MVIsp_Open(COM3, adjusted_baud[BAUD_9600], PARITY_NONE, WORDLEN8, STOPBITS1);
	if(se != MVI_SUCCESS){
	   printf("Unsuccessfully!\n");
		return 0;
	}
	else
	   printf("Successfully!\n");

	port_set |= 0x01;	// set to RS-232
//	port_set |= 0x02; // set to RS-485
//	port_set |= 0x04; // set to RS-422

	// Port 2
	printf("Open Communication Port 2 9600 8 1 None....");
	se = MVIsp_Open(COM4, adjusted_baud[BAUD_9600], PARITY_NONE, WORDLEN8, STOPBITS1);
	if(se != MVI_SUCCESS){
	   printf("Unsuccessfully!\n");
		return 0;
	}
	else
	   printf("Successfully!\n");

	port_set |= 0x08;	// set to RS-232
//	port_set |= 0x10; // set to RS-485
//	port_set |= 0x20; // set to RS-422

	// Port 3
	printf("Open Communication Port 3 9600 8 1 None....");
	se = MVIsp_Open(COM5, adjusted_baud[BAUD_9600], PARITY_NONE, WORDLEN8, STOPBITS1);
	if(se != MVI_SUCCESS){
	   printf("Unsuccessfully!\n");
		return 0;
	}
	else
	   printf("Successfully!\n");

	port_set |= 0x40;	// set to RS-232
//	port_set |= 0x80; // set to RS-485
//	eport_set = 0x01; // set to RS-422

//set interface configuration for upper ports (1 - 3); In this case RS-232
//         eport                   port
//                      P3  P3 P3 P2 P2 P2 P1 P1 P1  Port
// 7  6  5  4  3  2  1  0   7  6  5  4  3  2  1  0   Bit
// x  x  x  x  x  x  x  4   4  2  4  4  2  4  4  2
// x  x  x  x  x  x  x  2   8  3  2  8  3  2  8  3
// x  x  x  x  x  x  x  2   5  2  2  5  2  2  5  2

	OUTPORTB(SER_PORT_SET, port_set);
	OUTPORTB(PORT_LED_PORT, eport_set);

	// Set comm handshaking to none for each port
	MVIsp_SetHandshaking(COM2, HSHAKE_NONE);
	MVIsp_SetHandshaking(COM3, HSHAKE_NONE);
	MVIsp_SetHandshaking(COM4, HSHAKE_NONE);
	MVIsp_SetHandshaking(COM5, HSHAKE_NONE);

	// Set RTS line off for each port
	MVIsp_SetRTS(COM2, OFF);
	MVIsp_SetRTS(COM3, OFF);
	MVIsp_SetRTS(COM4, OFF);
	MVIsp_SetRTS(COM5, OFF);

	// Set DTR line on
	MVIsp_SetDTR(COM2, ON);
	MVIsp_SetDTR(COM3, ON);
	MVIsp_SetDTR(COM4, ON);
	MVIsp_SetDTR(COM5, ON);

	// Set module status LEDs for each port
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PCFG, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_FAULT, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_OFF);

	printf("\nVERSION INFORMATION:\n\n\
  %s\n\
  Copyright(c) 1999-2006, ProSoft Technology, Inc.\n\n\
  PRODUCT NAME CODE         : %s\n\
  SOFTWARE REVISION LEVEL   : %s\n\
  RUN NUMBER                : %s\n\n\
  SERIAL API VERSION        : %d.%d\n\
  ADM API VERSION           : %d.%d\n\n",
	"x102-ADM4",
	"ADM Serial Input Sample",
	"2.01",
	"10/25/2006",
	spverinfo.APISeries, spverinfo.APIRevision,
	adm_version.APIRevisionMajor, adm_version.APIRevisionMinor);

	printf("Press Esc to Exit.\n\n");

	return 1;
}

///////////////////////////////////////////////////////////////////////////////
//
// This function is used to shutdown the module's hardware and device drivers.
//
// Input : None.
// Output: The function will always return a value of 1.
//
///////////////////////////////////////////////////////////////////////////////

int shutdown_module(void)
{
	// set LEDs to shutdown configuration
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PCFG, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_FAULT, ADM_LED_ON);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_OFF);

	// close the comm ports
	printf("Closing Serial Port Driver....\n");
	MVIsp_Close(COM2);
	MVIsp_Close(COM3);
	MVIsp_Close(COM4);
	MVIsp_Close(COM5);

	return 1;
}

